//! Argument stuffs
use super::*;


mod error;
mod parse;

pub fn program_name() -> &'static str
{
    lazy_static!{
	static ref NAME: String = std::env::args().next().unwrap();
    }

    &NAME[..]
}

pub fn usage() -> !
{
    println!(r#"signing encryptor version {pkg_version} (format version {real_version})
 by {authour} with <3
 licensed with GPL v3 or later

Usage: {program} [OPTIONS...] [-e|-d|-a] [-] [<files...>]
Usage: {program} --generate RSA [--input <old key>] [--input-password[=<password>]] [--password[=<password>]] [--format BIN|TEXT|PEM] <output> [<public part output>]
Usage: {program} --generate AES [--input <old key>] [--input-password[=<password>]] [--password[=<password>]] [--format BIN|TEXT] <output>
Usage: {program} --stat [-P=<password>] [-] [-p]<key files...>
Usage: {program} --help

	-		Stop reading argument flags:
	-e		Encryption mode
	-d		Decryption mode
	-a		Autodetect mode (default)
OPTIONS:
	-t <inp> <out>	Process <input> to <output> (autodetect mode)
	-te <inp> <out>	Process <input> to <output> (encryption mode)
	-td <inp> <out>	Process <input> to <output> (decryption mode)
	-i		Process in-place for non-specific files (i.e. do not add or remove `.rae` file extension)

	-k <key>	Use this key (autodetect type)
	-kR <key>	Use this key (rsa private)
	-kr <key>	Use this key (rsa public)
	-ka <key>	Use this key (aes)
Other options for `-k`
	 -p		Read a password from stdin for this key
	 -P <password>	The next argument after the key file is a password for the key
	 -s		This key is also a signing key (or verifying for decrypt)
	 -S		This key is only a signing key (or verifying for decrypt, will fail on bad verification)

EXAMPLE:
	{program} -kRP key.rsa "super secret password" -ka key.aes -te input_file output_encrypted file
"#,
	     pkg_version = env!("CARGO_PKG_VERSION"),
	     real_version = CURRENT_VERSION,
	     authour = env!("CARGO_PKG_AUTHORS"),
    program = program_name());
    std::process::exit(1)
}

#[instrument]
/// Parse the program args into `Operation`.
pub async fn parse_args() -> Result<config::Operation, eyre::Report>
{
    parse::parse(std::env::args().skip(1)).await
        .with_suggestion(|| "Try passing `--help`")
}
